//////////////////////////////////////////////////
//  Module      : Sprite.h  Version 0.1
//  LastEdit    : 16.5.93
//  Description : Klasse fuer Sprites 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#ifndef _SPRITE_H
#define _SPRITE_H

#include <assert.h>

#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/xpm.h>

#include "Types.h"
#include "Point.h"
#include "Size.h"
#include "Rect.h"

//////////////////////////////////////////////////
// CSprite - Klasse fuer Sprites

class CSprite
{
  private:
    static uint m_nMaxId;               // aktuelle hoechste ID
    const  uint m_nSpriteId;            // jeweilige Sprite ID

    static GC m_gcand;                  // GC zum Kopieren der Sprite Maske 
    static GC m_gcor;                   // GC zum Kopieren des Sprite Image
    static GC m_gccopy;                 // GC zum Kopieren wenn keine Maske
                                        // vorhanden ist

  private:
    struct SpriteData
    {
      // Bilddaten
      Pixmap  *m_pixmaps;    // Sprite Bild 
      Pixmap  *m_masks;      // Sprite Masken
      XpmAttributes *m_pAttributes; // Attribute von XpmReadFileToPixmap()
      CSize    m_size;       // Sprite Groesse

      uint     m_nNumSequences;// Anzahl der Pixmaps eines Sprites

      // Position und Bewegung
      CPoint   m_pointCurPos;// aktuelle Position 
      CPoint   m_pointLastPos; // letzte Position 
      CSize    m_sizeDeltaLastPos; // Differenz alte neue Position 

      // Status
      bool     m_bActive;    // Sprite aktiv
      bool     m_bUpdate;    // Sprite neu Zeichnen
      bool     m_bOverlap;   // Ueberlagerung mit anderen Sprite
      bool     m_bMove;      // Sprite wird bewegt

      // Animation
      int      m_nAnimate;   // Animation - Anzahl, nach wievielen Frames
                             // eine Sequenz ausgetauscht wird. 
      uint     m_nCurSequence;// aktuelle Sequenz

      Display  *m_pDisplay;  // Display wegen XFreePixmap()
      Colormap  m_colormap;  // Wegen XFreeColors()

      uint     m_nReferenceCnt;// Referenzzaehler      

      SpriteData() : m_pixmaps(NULL), m_masks(NULL), 
                     m_pAttributes(NULL),
                     m_nNumSequences(0),
                     m_bActive(false), m_bUpdate(false), m_bOverlap(false),
                     m_bMove(false),
                     m_nAnimate(0), m_nCurSequence(0),
                     m_pDisplay(NULL),
                     m_colormap((Colormap)0),
                     m_nReferenceCnt(1) { }

      ~SpriteData()
      {
        int n;

        for (n = 0; n < m_nNumSequences; n++)
        {
          if (m_pixmaps != NULL && m_pixmaps[n] != (Pixmap)0)
            XFreePixmap(m_pDisplay, m_pixmaps[n]);
          if (m_masks != NULL && m_masks[n] != (Pixmap)0)
            XFreePixmap(m_pDisplay, m_masks[n]);
          if (m_pAttributes != NULL)
          {
            if (m_pAttributes[n].npixels > 0)
            {
              XFreeColors(m_pDisplay,
                m_colormap,
                m_pAttributes[n].pixels,
                m_pAttributes[n].npixels,
                0L);
            }
            XpmFreeAttributes(&m_pAttributes[n]);
          }
        }
        if (m_pixmaps != NULL)
          delete [] m_pixmaps; 
        if (m_masks != NULL)
          delete [] m_masks;
      }
    };

    // Pointer auf Daten
    SpriteData *m_pSpriteData;

    // sonstige Daten
    Widget   m_widget;       // widget
    Colormap m_colormap;     // Colormap 

    // Daten freigeben
    void Detach();

  public:
    CSprite(Widget widget, Colormap colormap = (Colormap)0);
    ~CSprite();

    // Aufgrund dynamischer Komponenten
    CSprite(const CSprite& sprite);
    const CSprite& operator= (const CSprite& sprite);

    // Sprite erzeugen
    bool Create(char ***chData, uint nNumSequences, 
      CPoint pos, int nAnimate);  

    // Sprite vorhanden
    bool Available() const;

    // Sprite Groesse
    int Width() const;
    int Height() const;

    // Sprite Position
    CPoint Position() const;
    void Position(CPoint pos);
    CPoint LastPosition() const;

    // Bewegung
    void ResetMoves();
    void Move(CSize sizeDeltaLastPos);
    CSize DeltaLastPos() const;
    bool IsMoving() const;

    // Sprite Status
    bool Active() const; 
    void Active(bool bState); 

    bool Update() const; 
    void Update(bool bState); 

    bool Overlap() const;
    void Overlap(bool bState);

    // Anzahl Frames, bis die naechste Sequenz eines Sprite gezeigt wird 
    // <m_nAnimate> <  0 Rueckwaertsanimation
    // <m_nAnimate> >  0 Forwaertsanimation
    // <m_nAnimate> == 0 Keine Animation
    int Animate() const; 
    void Animate(int nAnimate);
    
    // Setzen und Abfragen der aktuellen Sprite Sequenz
    int Sequence() const;
    void Sequence(int nSequence); 
    uint NumSequences() const; 
  
    // Sprite Zeichnen
    void Show(Drawable d, CPoint pos) const;

    // Sprite ID
    uint Id() const;

    // Umriss des Sprites
    CRect Extent() const;

    // Umriss der Bewegung des Sprites
    CRect MoveExtent() const;
};
//////////////////////////////////////////////////
// C S p r i t e : : A v a i l a b l e

inline bool CSprite::Available() const
{
  return (m_pSpriteData->m_pixmaps != NULL ? true : false);
}
//////////////////////////////////////////////////
// C S p r i t e : : W i d t h 

inline int CSprite::Width() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_pAttributes[0].width;
}
//////////////////////////////////////////////////
// C S p r i t e : : H e i g h t 

inline int CSprite::Height() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_pAttributes[0].height;
}
//////////////////////////////////////////////////
// C S p r i t e : : P o s i t i o n

inline CPoint CSprite::Position() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_pointCurPos;
}
//////////////////////////////////////////////////
// C S p r i t e : : P o s i t i o n

inline void CSprite::Position(CPoint pos)
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  if (m_pSpriteData->m_pointCurPos != pos)
  {
    m_pSpriteData->m_pointLastPos = m_pSpriteData->m_pointCurPos;
    m_pSpriteData->m_pointCurPos = pos;
    m_pSpriteData->m_sizeDeltaLastPos = 
      m_pSpriteData->m_pointCurPos - m_pSpriteData->m_pointLastPos;
    m_pSpriteData->m_bUpdate = true;
  }
}
//////////////////////////////////////////////////
// C S p r i t e : : L a s t P o s i t i o n

inline CPoint CSprite::LastPosition() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_pointLastPos;
}
//////////////////////////////////////////////////
// C S p r i t e : : R e s e t M o v e s

inline void CSprite::ResetMoves()
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  //m_pSpriteData->m_sizeDeltaLastPos = CSize(0,0);
  m_pSpriteData->m_bMove = false;
}
//////////////////////////////////////////////////
// C S p r i t e : : M o v e

inline void CSprite::Move(CSize sizeDeltaLastPos)
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  m_pSpriteData->m_sizeDeltaLastPos = sizeDeltaLastPos;
  m_pSpriteData->m_bUpdate = true;
  m_pSpriteData->m_bMove = true;
}
//////////////////////////////////////////////////
// C S p r i t e : : I s M o v i n g 

inline bool CSprite::IsMoving() const
{
  return m_pSpriteData->m_bMove;
}
//////////////////////////////////////////////////
// C S p r i t e : : D e l t a L a s t P o s 

inline CSize CSprite::DeltaLastPos() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_sizeDeltaLastPos;
}
//////////////////////////////////////////////////
// C S p r i t e : : A c t i v e

inline bool CSprite::Active() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_bActive;
}
//////////////////////////////////////////////////
// C S p r i t e : : A c t i v e

inline void CSprite::Active(bool bState) 
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  m_pSpriteData->m_bActive = bState;
}
//////////////////////////////////////////////////
// C S p r i t e : : U p d a t e 

inline bool CSprite::Update() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_bUpdate;
}
//////////////////////////////////////////////////
// C S p r i t e : : U p d a t e 

inline void CSprite::Update(bool bState) 
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  m_pSpriteData->m_bUpdate = bState;
};
//////////////////////////////////////////////////
// C S p r i t e : : O v e r l a p 

inline bool CSprite::Overlap() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_bOverlap;
}
//////////////////////////////////////////////////
// C S p r i t e : : O v e r l a p 

inline void CSprite::Overlap(bool bState) 
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  m_pSpriteData->m_bOverlap = bState;
}
//////////////////////////////////////////////////
// C S p r i t e : : A n i m a t e 

inline int CSprite::Animate() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_nAnimate;
}
//////////////////////////////////////////////////
// C S p r i t e : : A n i m a t e 

inline void CSprite::Animate(int nAnimate) 
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  m_pSpriteData->m_nAnimate = nAnimate;
}
//////////////////////////////////////////////////
// C S p r i t e : : S e q u e n c e 

inline int CSprite::Sequence() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_nCurSequence;
}
//////////////////////////////////////////////////
// C S p r i t e : : S e q u e n c e 

inline void CSprite::Sequence(int nSequence) 
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  if (nSequence >=  (int)m_pSpriteData->m_nNumSequences)
  {
    m_pSpriteData->m_nCurSequence = 0;
  }
  else if (nSequence < 0)
  {
    m_pSpriteData->m_nCurSequence = m_pSpriteData->m_nNumSequences - 1;
  }
  else
  {
    m_pSpriteData->m_nCurSequence = nSequence;
  }
}
//////////////////////////////////////////////////
// C S p r i t e : : N u m S e q u e n c e s 

inline uint CSprite::NumSequences() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);
  return m_pSpriteData->m_nNumSequences;
}

//////////////////////////////////////////////////
// C S p r i t e : : I d

inline uint CSprite::Id() const
{
  return m_nSpriteId;
}
//////////////////////////////////////////////////
// C S p r i t e : : E x t e n t

inline CRect CSprite::Extent() const
{
  assert(m_pSpriteData->m_pixmaps != NULL);

  return CRect(m_pSpriteData->m_pointCurPos.x,
               m_pSpriteData->m_pointCurPos.y,
               m_pSpriteData->m_pointCurPos.x + m_pSpriteData->m_size.cx,
               m_pSpriteData->m_pointCurPos.y + m_pSpriteData->m_size.cy);
}
//////////////////////////////////////////////////
// C S p r i t e : : M o v e E x t e n t

inline CRect CSprite::MoveExtent() const
{
  CRect rectReturn;
  CRect rectOldExtent(m_pSpriteData->m_pointLastPos.x,
    m_pSpriteData->m_pointLastPos.y,
    m_pSpriteData->m_pointLastPos.x + m_pSpriteData->m_size.cx,
    m_pSpriteData->m_pointLastPos.y + m_pSpriteData->m_size.cy);

  rectReturn.UnionRect(Extent(), rectOldExtent);
  return rectReturn;
}

#endif // _SPRITE_H
