//////////////////////////////////////////////////
//  Module      : SpriteAnimArea.C  Version 0.1
//  LastEdit    : 23.5.94
//  Description : Abstrakte Klasse zur Animation
//                von Sprites innerhalb der Motif 
//                DrawingArea 
//  Author      : T.Helbig
//////////////////////////////////////////////////
#include <X11/xpm.h>
#include <X11/Intrinsic.h>

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>

#include "SpriteAnimArea.h"

#include "Event.h" // EventHandler als C++Elementfunktion

//////////////////////////////////////////////////
// CSpriteAnimArea : : C S p r i t e A n i m A r e a 

CSpriteAnimArea::CSpriteAnimArea(Widget              wParent, 
                                 const char         *szName, 
                                 const char         **pBackgroundData, 
                                 const SpriteStructData *spriteObjects,
                                 uint                nNumObjects, 
                                 Colormap            colormap)
  : CBase(szName), 
    m_pSpriteAnim(NULL), 
    m_pSprites(NULL), 
    m_nNumSprites(0),
    m_pixmapBackground((Pixmap)0), 
    m_pTimer(NULL), 
    m_bGood(true)
{
  int nStatus;
  int n;
  Pixmap pixmapMask; 

  // Drawing Area erzeugen
  m_wBase = XtVaCreateWidget(szName, 
    xmDrawingAreaWidgetClass, wParent,
    NULL);

  // EventHandler zum Ermitteln der MousePosition registrieren
  AddEventHandler(m_wBase,
    PointerMotionMask, 
    False,
    &CSpriteAnimArea::MousePosEH,
    (XtPointer) NULL,
    (XtPointer) this);

  if (colormap == (Colormap) 0)
  {
    m_colormap = DefaultColormap(XtDisplay(wParent),
      DefaultScreen(XtDisplay(wParent)));
  }
  else
  {
    m_colormap = colormap;
  }

  // Anzahl der Sprite-Objekte 
  m_nNumSprites = nNumObjects;

  // Attribute setzen
  XtVaGetValues(m_wBase,
    XmNdepth, &m_xpmAttributes.depth,
    NULL);
  m_xpmAttributes.visual = DefaultVisual(XtDisplay(m_wBase),
    DefaultScreen(XtDisplay(m_wBase)));
  m_xpmAttributes.colormap = m_colormap;
  m_xpmAttributes.closeness = 40000;
  m_xpmAttributes.valuemask =
  XpmColormap | XpmReturnPixels | XpmSize | XpmCloseness | XpmDepth | XpmVisual;

  // Hintergrund erzeugen
  nStatus = XpmCreatePixmapFromData(XtDisplay(m_wBase),
    RootWindow(XtDisplay(m_wBase), DefaultScreen(XtDisplay(m_wBase))),
    (char**)pBackgroundData,
    &m_pixmapBackground,
    &pixmapMask,
    &m_xpmAttributes);

  // Konnte Hintergrund erzeugt werden ?
  if (nStatus != XpmSuccess)
  {
    if (nStatus == XpmNoMemory)
    {
      cerr << "Fehler in <CSpriteAnimArea::CSpriteAnimArea> "
           << "Kein Speicherplatz mehr vorhanden" << endl;
    }        
    else if (nStatus == XpmColorFailed)            
    {                                                
      cerr << "Fehler in <CSpriteAnimArea::CSpriteAnimArea> "
           << "Kann Farben nicht allokieren" << endl << endl;      
      cerr << "Versuche <xmsprite -c> !!!" << endl << endl;
    }                             
    else                                   
    {                                                  
      cerr << "Fehler in <CSpriteAnimArea::CreateIntern> "
           << "kann Hintergrundbild nicht erzeugen" << endl;
    }
   
    m_bGood = false;
  }

  // Wenn Maske vorhanden, dann loeschen, da fuer Hintergrund nicht benoetigt 
  if (pixmapMask)
  {
    XFreePixmap(XtDisplay(m_wBase), pixmapMask);
  }

  // Speicherplatz fuer Zeiger auf Sprite-Objekte allokieren
  m_pSprites = new CSprite* [nNumObjects];

  // Sprites erzeugen
  for (n = 0; n < nNumObjects; n++)
  {
    m_pSprites[n] = new CSprite(m_wBase, m_colormap);
    nStatus = m_pSprites[n]->Create(
      spriteObjects[n].m_pszSequenceNames,
      spriteObjects[n].m_nNumSequences,
      spriteObjects[n].m_pointStartPos,
      spriteObjects[n].m_nAnimation);

    if (!nStatus)
    {
      cerr << "Fehler in <CSpriteAnimArea::CSpriteAnimArea>"
           << " Kann Sprite nicht erzeugen" << endl;

      m_bGood = false;
    }
  }

  // Animations Objekt erzeugen
  m_pSpriteAnim = new CSpriteAnim(m_wBase, 
    CSize(m_xpmAttributes.width, 
          m_xpmAttributes.height),
          m_pixmapBackground);

  // Sprites zum Animations Objekt hinzufuegen
  for (n = 0; n < nNumObjects; n++)
  {
    m_pSpriteAnim->Add(*m_pSprites[n]);
  }

  // Timer erzeugen
  m_pTimer = new CSpriteTimer(m_wBase, m_pSpriteAnim, this);

  // Groesse der Drawing Area auf Hintergrundbild setzen
  XtVaSetValues(m_wBase,
    XmNheight, m_xpmAttributes.height,
    XmNwidth, m_xpmAttributes.width,
    NULL);

}
//////////////////////////////////////////////////
// CSpriteAnimArea : : C S p r i t e A n i m A r e a 

CSpriteAnimArea::~CSpriteAnimArea()
{
  int n;

  if (m_pSpriteAnim != NULL)
    delete m_pSpriteAnim;
  if (m_pSprites != NULL)
  {
    for (n = 0; n < m_nNumSprites; n++)
      delete m_pSprites[n];
    delete [] m_pSprites;
  }
  if (m_pixmapBackground != (Pixmap)0)
    XFreePixmap(XtDisplay(m_wBase), m_pixmapBackground);
  if (m_xpmAttributes.npixels > 0)
  {
    XFreeColors(XtDisplay(m_wBase),
      m_colormap,
      m_xpmAttributes.pixels,
      m_xpmAttributes.npixels,
      0L);
    XpmFreeAttributes(&m_xpmAttributes);
  }
  if (m_pTimer != NULL)
    delete m_pTimer;
}
//////////////////////////////////////////////////
// CSpriteAnimArea:: M o u s e P o s E H

void CSpriteAnimArea::MousePosEH(Widget    w, 
                                 XtPointer pClientData, 
                                 XEvent   *pEvent, 
                                 Boolean  *pFlag)
{
  m_pointMousePos = CPoint(pEvent->xmotion.x, pEvent->xmotion.y);
}
